"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const chalk_1 = tslib_1.__importDefault(require("chalk"));
const cli_ux_1 = tslib_1.__importDefault(require("cli-ux"));
const guruClient_1 = tslib_1.__importDefault(require("../lib/guruClient"));
const helpers_1 = require("../lib/helpers");
exports.default = async (cli) => {
    cli.command('find-replace <find> <replace> [filter]', 'Find and replace across guru articles (supports regex in find) with optional filtering')
        .option('-f, --force', 'Skip confirmation, BE CAREFUL WITH THIS OPTION!')
        .option('-i, --ignore-case', 'Ignore case in the finding regex string')
        .option('-C, --confirm', 'confirm making the changes (required to push modifications to guru)')
        .option('-w, --preview-width <size>', 'characters of text around the match when in preview mode (no --confirm)')
        .action(async (find, replace, filter, options) => {
        (0, helpers_1.checkAuth)();
        cli_ux_1.default.action.start('Finding cards based on filter');
        const cardsRaw = await guruClient_1.default.getAllCardsRaw();
        const filteredCards = (0, helpers_1.handleCardsFilter)(filter, cardsRaw, options, {
            raw: true,
        });
        let reFlags = 'g';
        if (options.ignoreCase)
            reFlags += 'i';
        const matchedCards = filteredCards.filter((card) => new RegExp(find, reFlags).test(card.content));
        if (!matchedCards.length) {
            cli_ux_1.default.action.stop(`❌ ${matchedCards.length} cards found matching "${find}"`);
            return;
        }
        cli_ux_1.default.action.stop(`✅ ${matchedCards.length} cards found matching "${find}"`);
        if (!options.confirm) {
            console.log('Preview:\n------------------------');
            for (const card of matchedCards) {
                console.log(`${chalk_1.default.bold.blueBright(card.preferredPhrase)} (${chalk_1.default.blue(`https://app.getguru.com/card/${card.slug}`)})`);
                const matches = [...card.content.matchAll(new RegExp(find, reFlags))];
                if (!matches.length)
                    throw new Error(`Unknown regex matching error occurred on card ${card.preferredPhrase}`);
                for (const match of matches) {
                    const matchText = match[0];
                    const matchIndex = match.index;
                    const SIZE_AROUND = options.previewWidth ? Number(options.previewWidth) : 40;
                    const beforeText = card.content.substring(matchIndex > SIZE_AROUND ? matchIndex - SIZE_AROUND : 0, matchIndex);
                    const afterText = card.content.substring(matchIndex + matchText.length, matchIndex + matchText.length + SIZE_AROUND >= card.content.length - 1
                        ? card.content.length - 1
                        : matchIndex + matchText.length + SIZE_AROUND);
                    console.log(`${beforeText}${chalk_1.default.bold.redBright(matchText)}${chalk_1.default.bold.green(replace)}${afterText}`);
                }
                console.log('');
            }
            console.log(`Rerun this same command with --confirm to push these changes to these ${matchedCards.length} Guru cards`);
            return;
        }
        if (!options.force) {
            const backup = await cli_ux_1.default.confirm('Would you like to backup card data before making changes? (RECOMMENDED) (Y/n)');
            if (backup) {
                await (0, helpers_1.backupCards)('./');
            }
            const confirm = await cli_ux_1.default.confirm(`This will replace all instances matching "${chalk_1.default.bold.redBright(find)}" with ${chalk_1.default.bold.green(replace)} in ${matchedCards.length} cards. Continue? (Y/n)`);
            if (!confirm)
                return;
        }
        for (const card of matchedCards) {
            card.content = card.content.replace(new RegExp(find, reFlags), replace);
            await guruClient_1.default.updateCardRaw(card);
        }
        console.log(`Successfully replaced "${chalk_1.default.bold.redBright(find)}" with "${chalk_1.default.bold.green(replace)}" across ${matchedCards.length} cards.`);
    });
};
