"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Guru = void 0;
const tslib_1 = require("tslib");
const axios_1 = tslib_1.__importDefault(require("axios"));
class Guru {
    constructor(_config) {
        this.email = _config.email;
        this.token = _config.token;
        this.httpConfig = {
            baseURL: 'https://api.getguru.com/api/v1',
            auth: {
                username: this.email,
                password: this.token,
            },
        };
    }
    async _request(method, url, data, axiosOptions) {
        try {
            const res = await (0, axios_1.default)(url, Object.assign(Object.assign({ method,
                data }, this.httpConfig), (axiosOptions !== null && axiosOptions !== void 0 ? axiosOptions : {})));
            return res;
        }
        catch (err) {
            if (err.response.status === 401) {
                throw new Error('Could not authenticate to Guru, check credentials and try again.');
            }
            throw err;
        }
    }
    _convertCardModel(cardRaw) {
        var _a, _b, _c;
        return {
            id: cardRaw.id,
            title: cardRaw.preferredPhrase,
            owner: (_b = (_a = cardRaw === null || cardRaw === void 0 ? void 0 : cardRaw.owner) === null || _a === void 0 ? void 0 : _a.email) !== null && _b !== void 0 ? _b : this.email,
            firstName: cardRaw.owner.firstName,
            lastName: cardRaw.owner.lastName,
            verifier: (_c = cardRaw === null || cardRaw === void 0 ? void 0 : cardRaw.cardVerifier) !== null && _c !== void 0 ? _c : '',
            collection: cardRaw.collection.name,
            boards: cardRaw.boards ? cardRaw.boards.map((board) => board.title) : [],
            content: cardRaw.content,
            verificationDate: cardRaw.nextVerificationDate,
            verificationState: cardRaw.verificationState,
            verificationInterval: cardRaw.verificationInterval,
            link: `https://app.getguru.com/card/${cardRaw.slug}`,
        };
    }
    async getGroupMembers(groupID) {
        const res = await this._request('GET', `groups/${groupID}/members`);
        return res.data;
    }
    async getVerifier(verifiers) {
        var _a, _b, _c, _d, _e, _f, _g;
        let verifier = '';
        if (((_a = verifiers[0]) === null || _a === void 0 ? void 0 : _a.type) === 'user-group') {
            const groupID = (_c = (_b = verifiers[0]) === null || _b === void 0 ? void 0 : _b.userGroup) === null || _c === void 0 ? void 0 : _c.id;
            if (!groupID)
                return '';
            const members = await this.getGroupMembers(groupID);
            if (!members.length)
                return '';
            verifier = (_e = (_d = members[0]) === null || _d === void 0 ? void 0 : _d.user) === null || _e === void 0 ? void 0 : _e.email;
        }
        else {
            verifier = (verifiers === null || verifiers === void 0 ? void 0 : verifiers.length) ? (_g = (_f = verifiers[0]) === null || _f === void 0 ? void 0 : _f.user) === null || _g === void 0 ? void 0 : _g.email : this.email;
        }
        return verifier;
    }
    async authenticated() {
        try {
            await this._request('GET', '/tasks/dashboard');
            return {
                authenticated: true,
                message: '',
            };
        }
        catch (err) {
            return {
                authenticated: false,
                message: err.message,
            };
        }
    }
    /**
     * Gets all Guru Cards
     * @returns Collection of Guru Cards
     */
    async getAllCardsRaw() {
        var _a, _b, _c;
        const allCards = [];
        // get first page of articles and push into final array
        let res = await this._request('POST', '/search/cardmgr', {});
        do {
            allCards.push(...res.data);
            const pageLink = (_b = (_a = res === null || res === void 0 ? void 0 : res.headers) === null || _a === void 0 ? void 0 : _a.link) === null || _b === void 0 ? void 0 : _b.match(/(?<=<)(.*)(?=>)/gs)[0];
            if (!pageLink)
                break;
            res = await this._request('POST', pageLink, {});
        } while ((_c = res === null || res === void 0 ? void 0 : res.headers) === null || _c === void 0 ? void 0 : _c.link);
        for (const card of allCards) {
            card.cardVerifier = await this.getVerifier(card.verifiers);
        }
        return allCards;
    }
    /**
     * Get all Guru Cards in a simpler format
     * @returns returns simple formatted guru cards
     */
    async getAllCards() {
        const allCards = await this.getAllCardsRaw();
        return allCards.map(this._convertCardModel);
    }
    async getCardsByTitle(cardTitle) {
        const { data: res } = await this._request('POST', '/search/cardmgr', {
            queryType: null,
            sorts: null,
            query: {
                nestedExpressions: [{ type: 'title', value: cardTitle, op: 'IS' }],
                op: 'AND',
                type: 'grouping',
            },
            collectionIds: [],
        });
        if (!res.length)
            throw new Error(`Unable to find card with title: ${cardTitle}`);
        const cards = res;
        return cards.map(this._convertCardModel);
    }
    async verifyCardByID(card) {
        const impersonationToken = await this.createImpersonationToken(card.verifier);
        await this._request('POST', '/cards/bulkop', {
            action: { type: 'verify-card' },
            items: { type: 'id', cardIds: [card.id] },
        }, impersonationToken
            ? {
                auth: {
                    username: card.verifier,
                    password: impersonationToken,
                },
            }
            : undefined);
        return {
            status: 'OK',
            message: `Success verifying ${card.title}`,
        };
    }
    /**
     * Verifies every card that is unverfied due to it being in an 'Expired' state
     * @returns success status
     */
    async getAllExpiredCardsRaw() {
        const expiredCards = [];
        const cards = await this.getAllCardsRaw();
        for (const card of cards) {
            if (card.verificationState === 'NEEDS_VERIFICATION') {
                const { data: cardDetail } = await this._request('GET', `/cards/${card.id}`);
                if (!cardDetail.verificationReasons.length ||
                    cardDetail.verificationReasons[0] === 'EXPIRED') {
                    expiredCards.push(card);
                }
            }
        }
        return expiredCards;
    }
    /**
     * Verifies every card that is in an unverified state
     * @returns success status
     */
    async getAllUnverifiedCardsRaw() {
        const unverifiedCards = [];
        const cards = await this.getAllCardsRaw();
        return cards.filter((c) => c.verificationState === 'NEEDS_VERIFICATION');
    }
    async updateCardRaw(cardRaw) {
        const verifierEmail = cardRaw.cardVerifier;
        if (!verifierEmail)
            throw new Error('Unable to find verifier email in card data');
        const { data: res } = await this._request('PUT', `/cards/${cardRaw.id}`, cardRaw, {
            auth: {
                username: verifierEmail,
                password: await this.createImpersonationToken(verifierEmail),
            },
        });
        return res;
    }
    async getAllCollections() {
        const { data: res } = await this._request('GET', '/collections');
        return res;
    }
    /**
     * impersonates a user.
     * @param userEmail user to impersonate
     * @returns impersonated user's token
     */
    async createImpersonationToken(userEmail) {
        if (userEmail === this.email)
            return this.token;
        const res = await (0, axios_1.default)({
            method: 'POST',
            url: `https://api.getguru.com/api/v1/api-tokens/users/${userEmail}`,
            data: {},
            auth: {
                username: this.email,
                password: this.token,
            },
        });
        return res.data.token;
    }
}
exports.Guru = Guru;
